# -*- coding: utf-8 -*-

import json
import logging
import socket
import sys
from collections import OrderedDict

from . import datetime_helper, settings


class JsonFormatter(logging.Formatter):
    def format(self, record):
        super().format(record)
        s = record.message
        if record.exc_text:
            if s[-1:] != "\n":
                s = s + "\n"
            s = s + record.exc_text

        # 计算method_name
        cls_name = ''
        func_name = record.funcName
        count = 40
        f = sys._getframe()
        while f.f_code.co_name != func_name and count > 0:
            f = f.f_back
            count -= 1
        func_frame = f if f.f_code.co_name == func_name else None
        caller = func_frame.f_locals.get('self', None) if func_frame else None

        try:
            if caller:
                if not hasattr(caller, '__name__'):
                    caller = caller.__class__
                cls_name = '%s.%s' % (caller.__module__, caller.__name__)
        except:
            pass
        finally:
            method_name = '{cls_name}.{func_name}'.format(cls_name=cls_name, func_name=func_name)
        msg = OrderedDict((
            ('@timestamp', datetime_helper.get_time_str()),
            ('level', record.levelname),
            ('project_name', settings.get('PROJECT_NAME')),
            ('project_version', settings.get('PROJECT_VERSION')),
            ("request_uri", getattr(record, 'request_uri', "")),
            ("trace_id", getattr(record, 'trace_id', "")),  # todo 通过thread获取trace_id?
            ("remote_ip", getattr(record, 'remote_ip', "")),
            ("local_ip", socket.gethostbyname(socket.gethostname())),
            ("logger_name", record.name),
            ("method_name", method_name),
            ("line_number", record.lineno),
            ("thread_name", record.threadName),
            ("message", s),
            ("stack_trace", record.stack_info),
        ))

        return json.dumps(msg)



# -*- coding: utf-8 -*-

import asyncio
import logging
import logging.config

import uvloop
from sanic.exceptions import NotFound
from sanic.response import redirect
from sanic_cors import CORS

from wings_sanic import settings, utils, serializers, registry, inspector
from wings_sanic.app import WingsSanic
from wings_sanic.swagger import swagger_blueprint

app = WingsSanic()
registry.set('app', app)


def start():
    """
    before start application, you need load your user_settings 
    by `wings_sanic.settings.load(**user_settings)`
    :return: 
    """
    if not settings.get('DEV'):
        logging.config.dictConfig(settings.get('LOG'))

    if settings.get('CORS'):
        CORS(app, automatic_options=True, supports_credentials=True)

    app.config.update(settings.working_settings)

    asyncio.set_event_loop_policy(uvloop.EventLoopPolicy())

    # when DEBUG==True, swagger document is available
    if settings.get('DEBUG'):
        app.blueprint(swagger_blueprint)

        @app.exception(NotFound)
        def handle_404_redirect(request, exception):
            return redirect('/swagger/')

    for bp_str in settings.get('BLUEPRINTS'):
        bp = utils.import_from_str(bp_str)
        app.blueprint(bp)

    @app.middleware('response')
    def log_response(request, response):
        logger = logging.getLogger('wings_sanic')
        full_path = request.path + ('?%s' % request.query_string if request.query_string else '')
        request_url = '{0} {1}'.format(request.method, full_path)
        extra = {
            'remote_ip': request.remote_addr,
            'request_uri': request_url
        }

        exception = getattr(response, 'exception', None)

        if exception:
            message = "request_url: {request_url}" \
                      "\nrequest_body: {body}" \
                      "\nstatus_code: {status_code}" \
                      "\nresponse_content: {response}" \
                .format(request_url=request_url, body=request.body,
                        response=response.body, status_code=response.status)
            if getattr(exception, 'code', 500) / 100 == 5:
                logger.error(message, extra=extra, exc_info=exception, stack_info=True)
            else:
                logger.warning(message, extra=extra, exc_info=exception)
        # 正常请求，记录少量信息
        else:
            message = '%s %s' % (request_url, response.status)
            logger.info(message, extra=extra)

    @app.exception(Exception)
    def handle_exception(request, exception):
        exception_handler = utils.import_from_str(settings.get('EXCEPTION_HANDLER'))
        response = exception_handler(request, exception)
        # set exception to response convenience for `log_response`
        setattr(response, 'exception', exception)
        return response

    @app.get('/ping/', response_serializer={'ping': serializers.StringField('Ping-Pong', required=True)})
    async def ping(request, *args, **kwargs):
        return {'ping': 'pong'}

    @app.listener('after_server_start')
    async def inspector_start_working(sanic, loop):
        registry.set('event_loop', loop)
        loop.call_soon(inspector.start)

    app.run(host="0.0.0.0", port=settings.get('HTTP_PORT'), workers=settings.get('WORKERS'),
            debug=settings.get('DEBUG'), access_log=settings.get('DEV'))



# -*- coding: utf-8 -*-
import datetime
import decimal
import inspect
from typing import Sequence, Mapping

from wings_sanic import datetime_helper


def instance_to_dict(instance):
    """
    Convert instance to dict
    """
    if not hasattr(instance, '__dict__'):
        return None
    data = {}
    for k, v in instance.__dict__.items():
        if k.startswith('__'):
            continue
        if k.startswith('_'):
            continue
        if callable(v):
            continue
        data[k] = v
    return data


def to_native(obj):
    """Convert obj to a richer Python construct. The obj can be anything
    """
    if obj is None:
        return None
    if isinstance(obj, (int, float, bool, datetime.datetime, datetime.date, decimal.Decimal)):
        return obj
    elif isinstance(obj, str):
        value = datetime_helper.parse_datetime(obj)
        if not value:
            value = datetime_helper.parse_date(obj)
        return value or obj

    if hasattr(obj, 'to_native') and callable(obj.to_native) \
            and len(inspect.signature(obj.to_native).parameters) == 1:
        return obj.to_native()

    if hasattr(obj, '__dict__'):
        obj = instance_to_dict(obj)

    if isinstance(obj, Sequence):
        return [to_native(item) for item in obj]
    if isinstance(obj, Mapping):
        return dict(
            (k, to_native(v)) for k, v in obj.items()
        )
    return obj


def to_primitive(obj):
    """Convert obj to a value safe to serialize.
    """
    if obj is None:
        return None
    if hasattr(obj, 'to_primitive') and callable(obj.to_primitive) \
            and len(inspect.signature(obj.to_primitive).parameters) == 1:
        return obj.to_primitive()
    data = to_native(obj)
    if isinstance(data, (int, float, bool, str)):
        return data
    if isinstance(data, datetime.datetime):
        return datetime_helper.get_time_str(obj)
    if isinstance(data, datetime.date):
        return datetime_helper.get_date_str(obj)
    if isinstance(data, Sequence):
        return [to_primitive(e) for e in data]
    elif isinstance(data, Mapping):
        return dict(
            (k, to_primitive(v)) for k, v in data.items()
        )
    return str(data)


def get_value(instance_or_dict, name, default=None):
    if isinstance(instance_or_dict, Mapping):
        return instance_or_dict.get(name, default)
    return getattr(instance_or_dict, name, default)


def cls_str_of_meth(meth):
    mod = inspect.getmodule(meth)
    cls = meth.__qualname__.split('.<locals>', 1)[0].rsplit('.', 1)[0]
    return '{0}.{1}'.format(mod.__name__, cls)


def cls_str_of_obj(obj):
    return '{0}.{1}'.format(obj.__class__.__module__, obj.__class__.__name__)


def cls_str_of_cls(cls):
    return '{0}.{1}'.format(cls.__module__, cls.__name__)


def meth_str(meth):
    return '{0}.{1}'.format(meth.__module__, meth.__qualname__)


def import_from_str(obj_path):
    module_name, obj_name = obj_path.rsplit('.', 1)
    module_meta = __import__(module_name, globals(), locals(), [obj_name])
    obj_meta = getattr(module_meta, obj_name)
    return obj_meta


# Removes all null values from a dictionary
def remove_nulls(dictionary, deep=True):
    return {
        k: remove_nulls(v, deep) if deep and type(v) is dict else v
        for k, v in dictionary.items()
        if v is not None
    }



# !/usr/bin/env python

import os
import sys

from setuptools import setup, find_packages

if sys.argv[-1] == 'publish':
    os.system("python setup.py sdist upload")
    sys.exit()

install_requires = [
    'six',
    'sanic',
    'Sanic-Cors',
]

setup(name='wings-sanic',
      version='0.1.4',
      description='The wings-sanic is a lightweight python framework aimed at making it as simple as possible to document your Sanic API with Swagger UI, Plus param validation and model serialization.',
      long_description='The wings-sanic is a lightweight python framework aimed at making it as simple as possible to document your Sanic API with Swagger UI, Plus param validation and model serialization.',
      author='SongTao',
      author_email='songtao@klicen.com',
      url='https://github.com/songtao-git/wings-sanic',
      packages=find_packages(),
      install_requires=install_requires,
      classifiers=[
          'Development Status :: 5 - Production/Stable',
          'Operating System :: MacOS',
          'Operating System :: POSIX :: Linux',
          'Topic :: System :: Software Distribution',
          'License :: OSI Approved :: MIT License',
          'Programming Language :: Python :: 3.5',
          'Programming Language :: Python :: 3.6',
          'Programming Language :: Python :: 3.7',
      ],
      include_package_data=True,
      )



# -*- coding: utf-8 -*-

import datetime
import re

ZERO_TIME_DELTA = datetime.timedelta(0)
LOCAL_TIME_DELTA = datetime.timedelta(hours=8)  # 本地时区偏差


class UTC(datetime.tzinfo):
    def utcoffset(self, dt):
        return ZERO_TIME_DELTA

    def dst(self, dt):
        return ZERO_TIME_DELTA


class LocalTimeZone(datetime.tzinfo):
    def utcoffset(self, dt):
        return LOCAL_TIME_DELTA

    def dst(self, dt):
        return ZERO_TIME_DELTA

    def tzname(self, dt):
        return '+08:00'


# singleton
UTC = UTC()
LocalTimeZone = LocalTimeZone()

date_re = re.compile(
    r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})$'
)

datetime_re = re.compile(
    r'(?P<year>\d{4})-(?P<month>\d{1,2})-(?P<day>\d{1,2})'
    r'[T ](?P<hour>\d{1,2}):(?P<minute>\d{1,2})'
    r'(?::(?P<second>\d{1,2})(?:\.(?P<microsecond>\d{1,6})\d{0,6})?)?'
    r'(?P<tzinfo>Z|[+-]\d{2}(?::?\d{2})?)?$'
)


def parse_date(value):
    """Parses a string(ISO_8601) and return a datetime.date.

    Raises ValueError if the input is well formatted but not a valid date.
    Returns None if the input isn't well formatted.
    """
    match = date_re.match(value)
    if match:
        kw = {k: int(v) for k, v in match.groupdict().items()}
        return datetime.date(**kw)


def parse_datetime(value):
    """Parses a string(ISO_8601) and return a datetime.datetime base UTC,
    or parse datetime.datetime base other timezone and return a datetime.datetime base UTC timezone
    """
    if isinstance(value, datetime.datetime):
        if not value.tzinfo:
            value = value.replace(tzinfo=LocalTimeZone)
        return value.astimezone(UTC)

    match = datetime_re.match(value)
    if match:
        kw = match.groupdict()
        if kw['microsecond']:
            kw['microsecond'] = kw['microsecond'].ljust(6, '0')
        tzinfo = kw.pop('tzinfo')
        tz = UTC
        offset = 0
        if tzinfo == 'Z':
            offset = 0
        elif tzinfo is not None:
            offset_mins = int(tzinfo[-2:]) if len(tzinfo) > 3 else 0
            offset = 60 * int(tzinfo[1:3]) + offset_mins
            if tzinfo[0] == '-':
                offset = -offset
        else:
            tz = LocalTimeZone
        kw = {k: int(v) for k, v in kw.items() if v is not None}
        kw['tzinfo'] = tz
        dt = datetime.datetime(**kw)
        dt += datetime.timedelta(minutes=offset)
        return dt.astimezone(UTC)


def convert_zone(dt: datetime.datetime, tz_to=UTC, tz_default=LocalTimeZone):
    """
    :param dt:
    :param tz_to: 转换后的目标时区
    :param tz_default: dt无时区信息时的默认时区
    :return:
    """
    if not hasattr(dt, 'tzinfo'):
        return dt
    if not dt.tzinfo:
        dt = dt.replace(tzinfo=tz_default)
    return dt.astimezone(tz_to)


def get_utc_time(dt: datetime.datetime = None, tz_default=LocalTimeZone):
    """
    :param dt: 为None时，返回当前时间
    :param tz_default: dt无时区信息时的默认时区
    :return:
    """
    if dt is None:
        dt = datetime.datetime.now()
    return convert_zone(dt, tz_default=tz_default)


def get_local_time(dt: datetime.datetime = None, tz_default=LocalTimeZone):
    """
    :param dt: 为None时，返回当前时间
    :param tz_default: dt无时区信息时的默认时区
    :return:
    """
    if dt is None:
        dt = get_utc_time()
    return convert_zone(dt, LocalTimeZone, tz_default)


def now():
    return get_utc_time()


def today():
    return datetime.date.today()


def get_time_str(dt: datetime.datetime = None, tz_default=LocalTimeZone):
    """
    :param dt: 为None时，返回当前时间
    :param tz_default: dt无时区信息时的默认时区
    :return:
    """
    if not dt:
        dt = datetime.datetime.now()
    dt = convert_zone(dt, tz_default=tz_default)
    time_str = dt.isoformat().split('+')[0]
    return time_str + 'Z'


def get_date_str(dt: datetime.date = None):
    """
    :param dt: 为None时，返回当前日期
    :return:
    """
    if not dt:
        dt = datetime.date.today()
    return dt.strftime('%Y-%m-%d')


def from_timestamp(timestamp):
    """
    :param timestamp:
    :return:
    """
    dt = datetime.datetime.fromtimestamp(timestamp)
    return get_utc_time(dt)


if __name__ == '__main__':
    print(get_utc_time(datetime.datetime.now()))
    print(get_local_time().strftime('%Y-%m-%d %H:%M:%S'))
    print(from_timestamp(1543675109).timestamp())
